/*
  Copyright Jesus Perez <jesusprubio gmail com>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

// https://developer.shodan.io/api/exploits/rest

const utils = require('./utils');

// Optional parameters for each function.
const optionalParams = {
  search: ['facets', 'page'],
  count: ['facets'],
};
// Part of all urls.
const apiSection = 'exploits';


function req(subPath, query, key, opts = {}) {
  let partialUrl = `/api/${subPath}?`;

  if (!query) { return Promise.reject(utils.createErrMandatory('query')); }

  partialUrl = utils.addParam(partialUrl, 'query', query);
  partialUrl = utils.addOptionals(partialUrl, optionalParams[subPath], opts);

  return utils.apiRequest(apiSection, partialUrl, key, opts.timeout);
}


module.exports.search = (query, key, opts = {}) => req('search', query, key, opts);


module.exports.count = (query, key, opts = {}) => req('count', query, key, opts);
